/**
 * MonFermenters.cpp is part of bmsapp.
 *
 * bmsapp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * bmsapp is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "MonFermenters.h"
#include "DetailFermenter.h"
#include "MainWindow.h"
#include "config.h"



/*
 * Build the table and buttons on the mainscreen.
 * Don't use a ui file, do it dynamicly.
 */
MonFermenters::MonFermenters(QWidget *parent) : QDialog(parent)
{
#ifdef DEBUG_MONITOR
    qDebug() << "MonFermenters start";
#endif

    gridLayout = new QGridLayout(this);
    gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
    tableFermenters = new QTableWidget(this);
    tableFermenters->setObjectName(QString::fromUtf8("tableFermenters"));
    tableFermenters->setEnabled(true);
    QSizePolicy sizePolicy(QSizePolicy::Fixed, QSizePolicy::Expanding);
    sizePolicy.setHorizontalStretch(0);
    sizePolicy.setVerticalStretch(0);
    sizePolicy.setHeightForWidth(tableFermenters->sizePolicy().hasHeightForWidth());
    tableFermenters->setSizePolicy(sizePolicy);
    tableFermenters->setMinimumSize(QSize(1054, 0));
    gridLayout->addWidget(tableFermenters, 0, 0, 1, 1);

    groupBox = new QGroupBox(this);
    groupBox->setObjectName(QString::fromUtf8("groupBox"));
    groupBox->setEnabled(true);
    groupBox->setFlat(false);
    horizontalLayout = new QHBoxLayout(groupBox);
    horizontalLayout->setSpacing(6);
    horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
    horizontalLayout->setContentsMargins(0, 0, 0, 0);
    quitButton = new QPushButton(groupBox);
    quitButton->setObjectName(QString::fromUtf8("quitButton"));
    quitButton->setMinimumSize(QSize(80, 24));
    quitButton->setText(tr("Quit"));
    QIcon icon;
    icon.addFile(QString::fromUtf8(":icons/silk/door_out.png"), QSize(), QIcon::Normal, QIcon::Off);
    quitButton->setIcon(icon);
    horizontalLayout->addWidget(quitButton, 0, Qt::AlignCenter);
    gridLayout->addWidget(groupBox, 1, 0, 1, 1);

    connect(quitButton, SIGNAL(clicked()), parent, SLOT(fromMonFermenters()));
    connect(this, SIGNAL(setStatus(QString)), parent, SLOT(statusMsg(QString)));
    connect(parent, SIGNAL(updateFermenters(QString)), this, SLOT(refreshFermenters(QString)));
    emit refreshTable();
}


void MonFermenters::refreshTable()
{
    QTableWidgetItem *item;

    QSqlQuery query("SELECT record,node,alias,online,beercode,beername,mode,air_temperature,beer_temperature FROM mon_fermenters ORDER BY node,alias");
    const QStringList labels({tr("Node"), tr("Unit"), tr("Status"), tr("Mode"), tr("Beer"), tr("Air"), tr("Beer"), tr("Details")});

    this->tableFermenters->setColumnCount(8);
    this->tableFermenters->setColumnWidth(0, 100);	/* Node		*/
    this->tableFermenters->setColumnWidth(1, 100);	/* Alias	*/
    this->tableFermenters->setColumnWidth(2, 100);	/* Status	*/
    this->tableFermenters->setColumnWidth(3, 100);	/* Mode		*/
    this->tableFermenters->setColumnWidth(4, 360);	/* Beer		*/
    this->tableFermenters->setColumnWidth(5,  90);	/* Air temp	*/
    this->tableFermenters->setColumnWidth(6,  90);	/* Beer temp	*/
    this->tableFermenters->setColumnWidth(7,  90);	/* Edit button	*/
    this->tableFermenters->setRowCount(query.size());
    this->tableFermenters->setHorizontalHeaderLabels(labels);
    this->tableFermenters->verticalHeader()->hide();
    /* Set the widget size to 1054 x 575 in the ui. */

    query.first();
    for (int i = 0 ; i < query.size() ; i++ ) {
	this->tableFermenters->setItem(i, 0, new QTableWidgetItem(query.value("node").toString()));
	this->tableFermenters->setItem(i, 1, new QTableWidgetItem(query.value("alias").toString()));

	if (query.value("online").toInt()) {
            item = new QTableWidgetItem(QString("Ok"));
        } else {
            item = new QTableWidgetItem(QString("Offline"));
            item->setForeground(QBrush(QColor(Qt::red)));
        }
        item->setTextAlignment(Qt::AlignCenter|Qt::AlignVCenter);
        this->tableFermenters->setItem(i, 2, item);

	item = new QTableWidgetItem(query.value("beercode").toString()+" - "+query.value("beername").toString());
	this->tableFermenters->setItem(i, 4, item);

	if (query.value("online").toInt()) {
	    item = new QTableWidgetItem(query.value("mode").toString());
	    item->setTextAlignment(Qt::AlignCenter|Qt::AlignVCenter);
	    this->tableFermenters->setItem(i, 3, item);
            item = new QTableWidgetItem(QString("%1°C").arg(query.value("air_temperature").toDouble(), 4, 'f', 3, '0'));
            item->setTextAlignment(Qt::AlignCenter|Qt::AlignVCenter);
            this->tableFermenters->setItem(i, 5, item);
	    item = new QTableWidgetItem(QString("%1°C").arg(query.value("beer_temperature").toDouble(), 4, 'f', 3, '0'));
            item->setTextAlignment(Qt::AlignCenter|Qt::AlignVCenter);
            this->tableFermenters->setItem(i, 6, item);
	} else {
	    this->tableFermenters->setItem(i, 3, new QTableWidgetItem(QString("")));
	    this->tableFermenters->setItem(i, 5, new QTableWidgetItem(QString("")));
	    this->tableFermenters->setItem(i, 6, new QTableWidgetItem(QString("")));
	}

	/* Add the Edit button */
	QWidget* pWidget = new QWidget();
	QPushButton* btn_edit = new QPushButton();
	btn_edit->setObjectName(QString("%1").arg(query.value("record").toString()));	/* Send record with the button */
	btn_edit->setText(tr("Details"));
	connect(btn_edit, SIGNAL(clicked()), this, SLOT(on_editButton_clicked()));
	QHBoxLayout* pLayout = new QHBoxLayout(pWidget);
	pLayout->addWidget(btn_edit);
	pLayout->setContentsMargins(5, 0, 5, 0);
	pWidget->setLayout(pLayout);
	this->tableFermenters->setCellWidget(i, 7, pWidget);
	query.next();
    }
    emit setStatus(QString(tr("Total items: %1")).arg(query.size()));
}


MonFermenters::~MonFermenters() {}


void MonFermenters::refreshFermenters(QString data)
{
    emit refreshTable();
    emit updateFermenter(data);
}


void MonFermenters::edit(int recno)
{
    DetailFermenter dialog(recno, this);
    dialog.setModal(true);
    dialog.exec();
}


void MonFermenters::on_editButton_clicked()
{
    QPushButton *pb = qobject_cast<QPushButton *>(QObject::sender());
    int recno = pb->objectName().toInt();
    edit(recno);
}


